/**************************************************************************************************

Copyright (c) 2007-2014 Tsunehiko Kato  and  4D2U Project, National Astronomical Observatory of Japan

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

ȉɒ߂ɏ]A{\tgEFAъ֘Ãt@Ciȉu\tgEFAvj̕擾
邷ׂĂ̐lɑ΂A\tgEFA𖳐ɈƂ𖳏ŋ܂Bɂ́A\tgEFA̕
gpAʁAύXAAfځAЕzATuCZXA/܂͔̔錠Aу\tgEFA
񋟂鑊ɓƂ錠Ɋ܂܂܂B

L̒쌠\і{\A\tgEFÂׂĂ̕܂͏dvȕɋLڂ̂Ƃ
B

\tgEFÁû܂܁vŁAł邩Öقł邩킸A̕ۏ؂Ȃ񋟂܂B
łۏ؂Ƃ́AiA̖ړIւ̓KAьNQɂĂ̕ۏ؂܂݂܂AɌ
̂ł͂܂B ҂܂͒쌠҂́A_sׁAs@sׁA܂͂ȊOł낤ƁA\t
gEFAɋN܂͊֘AA邢̓\tgEFA̎gp܂͂̑̈ɂĐ؂̐A
QȂ̋`ɂĉ̐ӔCȂ̂Ƃ܂B
***************************************************************************************************/
#include  <windows.h>
#include  <stdio.h>
#include  "../common/graphics/gl_common.h"
#include  "../common/graphics/gltext.h"
#include  "../common/math/vector3.h"
#include  "../common/astro/astro_const.h"
#include  "../common/graphics/topograph.h"
#include  "mtk_place_name_earth_mars.h"
#include  "../common/graphics/image.h"
#include  "mtk_colors.h"




//--  Place object
const char place_object[PLACE_OBJ_N][100] = {"EARTH", "MOON", "MARS"};



//--  Place key
enum {
	PLACE_KEY_DATA,
	PLACE_KEY_OBJECT,
	PLACE_KEY_TYPE,
	PLACE_KEY_N
};

const char place_key[PLACE_KEY_N][100] = {"", "OBJECT", "TYPE"};



//-- Objects
static vector<PlaceType> vPt;
static PlaceName vPn[PLACE_OBJ_N];



const char*
read_one(const char *p, char *buf)
{
	// skip white space
	while (isspace(*p)) {
		p++;
	}

	if (*p != '\"') {	// normal cases
		char ch;

		while (!isspace(ch = *p) && ch != ',' && ch != '\0')  {
			*buf = ch;
			p++;
			buf++;
		}
		*buf = '\0';
	}
	else {				// quoted string
		char ch;

		p++;
		while ((ch = *p) != '\"' && ch != '\0')  {
			*buf = ch;
			p++;
			buf++;
		}
		*buf = '\0';
	}

	if (*p != '\0') {
		p++;
	}

	if (*p == ',') {
		p++;
	}

	return p;
}



int
get_place_key(const char *str, const char **pos)
{
	char ch;

	int i = 0;
	while ((ch=str[i]) != '\0' && ch != ':') {
		i++;
	}

	if (ch == '\0') {
		return  PLACE_KEY_DATA;
	}

	for (int j = 0; j < PLACE_KEY_N; j++) {
		if (_strnicmp(str, place_key[j], i) == 0) {
			*pos = &str[i+1];
			return j;
		}
	}

	return  PLACE_KEY_DATA;
}



void
read_place_object(const char *str, int *obj_no)
{
	for (int i = 0; i < PLACE_OBJ_N; i++) {
		int n;
		n = strlen(place_object[i]);

		if (_strnicmp(str, place_object[i], n) == 0) {
			*obj_no = i;
			return;
		}
	}
}



bool
read_place_type(const char *str)
{
	char buf[500];
	const char *p;
	int type_no;
	PlaceType pt;


	// type no
	p = str;
	p = read_one(p, buf);
	if (sscanf(buf, "%d", &type_no) != 1) {
		return false;
	}

	// size
	p = read_one(p, buf);
	if (sscanf(buf, "%e", &pt.sz) != 1) {
		return false;
	}

	// R1
	p = read_one(p, buf);
	if (sscanf(buf, "%e", &pt.R1) != 1) {
		return false;
	}

	// R2
	p = read_one(p, buf);
	if (sscanf(buf, "%e", &pt.R2) != 1) {
		return false;
	}

	// col1
	p = read_one(p, buf);
	if (!is_RGBAref(buf)) {
		return false;
	}
	else {
		pt.col1 = get_RGBA_from_RGBAref(buf);
	}

	// col2
	p = read_one(p, buf);
	if (!is_RGBAref(buf)) {
		return false;
	}
	else {
		pt.col2 = get_RGBA_from_RGBAref(buf);
	}



	if (type_no < vPt.size()) {
		vPt[type_no] = pt;
	}
	else {
		while (vPt.size() <= type_no) {
			vPt.push_back(pt);
		}
	}

	return true;
}


bool
read_place_data(const char *str, PlaceNameItem &pni)
{
	char buf[500];
	const char *p;


	// type
	p = str;
	p = read_one(p, buf);
	if (sscanf(buf, "%d", &pni.ty) != 1) {
		return false;
	}

	// lon
	p = read_one(p, buf);
	if (sscanf(buf, "%e", &pni.lon) != 1) {
		return false;
	}

	// lat
	p = read_one(p, buf);
	if (sscanf(buf, "%e", &pni.lat) != 1) {
		return false;
	}

	// align
	p = read_one(p, buf);
	if (sscanf(buf, "%c", &pni.align) != 1) {
		return false;
	}

	// double-quoted name E
	p = read_one(p, pni.name_E);

	// double-quoted name J
	p = read_one(p, pni.name_J);

	if (pni.ty >= vPt.size()) {
		return false;
	}

	PlaceType& pt = vPt[pni.ty];
	pni.size = pt.sz;
	pni.R1 = pt.R1;
	pni.R2 = pt.R2;
	pni.col1 = pt.col1;
	pni.col2 = pt.col2;
	pni.bDispLabel = true;

	return true;
}


void
load_place_name(const char *fn)
{
	FILE *fp;
	char buf[500];
	PlaceNameItem pni;


	fp = fopen(fn, "rt");
	if (fp == NULL) return;

	for (int i = 0; i < PLACE_OBJ_N; i++) {
		vPn[i].clear();
	}


	int obj_no = PLACE_OBJ_EARTH;
	while (fgets(buf, 500, fp) != NULL) {
		int key;
		const char *p;
		bool bQuote = false;


		// comment out
		int n = strlen(buf);
		for (int i = 0; i < n; i++) {
			if (buf[i] == '\"') {
				if (bQuote) {
					bQuote = false;
				}
				else {
					bQuote = true;
				}
				continue;
			}
			if (buf[i] == '%' && !bQuote) {
				buf[i] = '\0';
				break;
			}
		}


		key = get_place_key(buf, &p);
		switch (key) {
		case PLACE_KEY_OBJECT:
			read_place_object(p, &obj_no);
			break;
		case PLACE_KEY_TYPE:
			read_place_type(p);
			break;
		case PLACE_KEY_DATA:
			if (read_place_data(buf, pni)) {
				vPn[obj_no].add_place(pni);
			}
			break;
		default:
			break;
		}
	}

	fclose(fp);
}



PlaceName&
get_place_name(int place_obj)
{
	if (place_obj < 0)  place_obj = 0;
	if (place_obj >= PLACE_OBJ_N)  place_obj = PLACE_OBJ_N-1;

	return vPn[place_obj];
}
