/**************************************************************************************************

Copyright (c) 2007   Tsunehiko Kato  and  4D2U Project, National Astronomical Observatory of Japan

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

ȉɒ߂ɏ]A{\tgEFAъ֘Ãt@Ciȉu\tgEFAvj̕擾
邷ׂĂ̐lɑ΂A\tgEFA𖳐ɈƂ𖳏ŋ܂Bɂ́A\tgEFA̕
gpAʁAύXAAfځAЕzATuCZXA/܂͔̔錠Aу\tgEFA
񋟂鑊ɓƂ錠Ɋ܂܂܂B

L̒쌠\і{\A\tgEFÂׂĂ̕܂͏dvȕɋLڂ̂Ƃ
B

\tgEFÁû܂܁vŁAł邩Öقł邩킸A̕ۏ؂Ȃ񋟂܂B
łۏ؂Ƃ́AiA̖ړIւ̓KAьNQɂĂ̕ۏ؂܂݂܂AɌ
̂ł͂܂B ҂܂͒쌠҂́A_sׁAs@sׁA܂͂ȊOł낤ƁA\t
gEFAɋN܂͊֘AA邢̓\tgEFA̎gp܂͂̑̈ɂĐ؂̐A
QȂ̋`ɂĉ̐ӔCȂ̂Ƃ܂B
***************************************************************************************************/
#include  "mtk_spacecraft_model.h"
#include  <windows.h>
#include  <stdio.h>
#include  "../common/graphics/gl_common.h"
#include  "../common/astro/astro_const.h"




bool
spacecraft::load_path(const char* filename)
{
	FILE *fp;
	if ((fp = fopen(filename, "rb")) == NULL)  return  false;


	path.clear();
	while (true) {
		spacecraft_path_data dat;
		
		int count;
		count = fread((void*)&dat, sizeof(double), 4, fp);
		if (count != 4)  break;

        dat.pos *= _1AU_pc;	// Pʂ AU  pc ɂ

		path.push_back(dat);
	}
	fclose(fp);

	return  true;
}


// w肵ŋO`Ă邩
bool
spacecraft::is_available_time(const double& t) const
{
	int i1 = 0;
	int i2 = path.size()-1;

	return  (t >= path[0].t || t <= path[path.size()-1].t);
}

double
spacecraft::get_t1() const
{
	if (path.size() == 0)  return 0.0;

	return  path[0].t;
}

double
spacecraft::get_t2() const
{
	if (path.size() == 0)  return 0.0;

	return  path[path.size()-1].t;
}


// w莞̈ʒu𓾂
vector3d
spacecraft::get_pos(const double& t)  const
{
	int i1 = 0;
	int i2 = path.size()-1;

	if (t <= path[i1].t)	{ return path[i1].pos; }
	if (t >= path[i2].t)	{ return path[i2].pos; }

	while (i2-i1 > 1) {
		int im = (i2+i1)/2;

		double tm;
		tm = path[im].t;
		if (tm==t)	{ return path[im].pos; }

		if (t < tm)	{ i2 = im; }
		else		{ i1 = im; }
	}

	// Ԃ
	double t1, t2, s;
	vector3d  pos1, pos2;
	t1   = path[i1].t;
	t2   = path[i2].t;
	pos1 = path[i1].pos;
	pos2 = path[i2].pos;
	s = (t-t1)/(t2-t1);

	return  (1.0-s) * pos1 + s * pos2;
}



// pX̕\
void
spacecraft::draw_path(const mtk_draw_struct& ds, const double& t, const vector3d& camera_pos, const RGBf& col, float af) const
{
	if (!bDrawPath)  return;

	if (t < path[0].t)  return;

	int idx = 0;
	vector3d pos;


	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glColor4f(col.r, col.g, col.b, af);
	
	const float lw = 1.5 * ds.line_width_fac / (ds.rad_per_pixel / 0.001);
	glLineWidth(lw);

	glBegin(GL_LINE_STRIP);
		while (idx < path.size()) {
			const spacecraft_path_data&  dat = path[idx];
			if (t < dat.t)  break;

			pos = dat.pos - camera_pos;

			glVertex3dv( pos.get_pointer() );
			idx++;
		}

		pos = get_pos(t)  - camera_pos;
		glVertex3dv( pos.get_pointer() );
	glEnd();
	glPopAttrib();
}




// f̕\
void
spacecraft::draw_model() const
{
	if (!pModel)  return;

	glPushMatrix();
		const float		scl = 3 * pModel->get_scale();
		const vector3f	r0  = pModel->get_r0();
		glScalef(scl,scl,scl);
		glTranslatef(r0.x, r0.y, r0.z);

		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
		pModel->draw();
		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
	glPopMatrix();
}


// x̕\
void
spacecraft::draw_label(wgl_font& fnt, bool bEng) const
{
	const char* text = (bEng)  ?  bszName.text_E  :  bszName.text_J;
	fnt.draw_text(text);
}


//
void
spacecraft::set(const char* name_E, const char* name_J, glModel3D* _pModel)
{
	bilingual name;
	name.text_E = name_E;
	name.text_J = name_J;
	set_name(name);

	set_model(_pModel);
}

